%% GENERAL INFORMATION
% Optimization module using the Dijkstra's shortest path algorithm
% to compute the (near) optimal routing scheme.
%
% Author: Piotr Jaglarz (pjaglarz@student.agh.edu.pl)
% Date: 04.2014

%% INPUT

% % Clear the environment
% 
% 	close all;
%     clear all;
% 	clc;

function last_optimization( protection, demands, distance, index, k_max )
% Read the complete environment snapshot from a .mat file

    gen_input_data( distance );
    clearvars -except protection demands index k_max;
    load( 'input' );
    
%% MODIFIED YAGED'S ALGORITHM
% (Pioro --> Algorithm 5.12 with the proposed modification 5.6.2)

    k = 1;                                      % Iteration counter
	F = Inf;                                    % Global cost
	F_new = Inf;                                % Global cost -> new value
	Pd = zeros( length( d ), length( A ) );     % Shortest paths for the corresponding traffic demands
    backup_paths_DP = zeros( length( d ), length( A ) );
    backup_paths_DL = zeros( length( A ), length( A ) );
    backup_paths_SP = zeros( length( d ), length( A ) );
    backup_paths_SL = zeros( length( A ), length( A ) );
    
    while ( 1 )
        
		% For each traffic demand, find the shortest path and increase the load
		% of the corresponding links
    	y = zeros( length( A ), 1 );                        % Link loads
        y_DL = zeros( length( A ), 1 );
        y_SL = zeros( length( A ), 1 );
        y3_SP = cell( length( A ), 1 );
        y3_SP( : ) = { 0 };
        y3_SL = cell( length( A ), 1 );
        y3_SL( : ) = { 0 };
        y2 = zeros( length( A ), 1 );
        	
        for i = 1 : length( d )
            protection = demands( i );
            if protection == 0 || protection == 2 || protection == 4
                [ cost, path ] = dijkstra( A, links, y, d( i, 1 ), d( i, 2 ), d( i, 3 ) );
            elseif protection == 1 || protection == 3
                [ cost, path, backup ] = suurballe( A, links, y, d( i, 1 ), d( i, 2 ), d( i, 3 ) );
            end

            if cost == Inf
				warning( 'There is no path between nodes %d and %d.\n', d( i, 1 ), d( i, 2 ) );
            else
                Pd( i, : ) = zeros( 1, length( A ) );                   % Remove the previous shortest path entry
                for m = 1 : length( path )
                    y( path( m ) ) = y( path( m ) ) + d( i, 3 );        % Increase the link load
                    Pd( i, m ) = path( m );                 			% Update the shortest path for the demand (link predecessor sequence)
                    if protection == 2
                        y_DL( path( m ) ) = y_DL( path( m ) ) + d( i, 3 );
                    end
                    if protection == 4
                        y_SL( path( m ) ) = y_SL( path( m ) ) + d( i, 3 );
                    end
                end
                
                if protection == 1
                    backup_paths_DP( i, : ) = zeros( 1, length( A ) );
                    for m = 1 : length( backup )
                        y( backup( m ) ) = y( backup( m ) ) + d( i, 3 );    % optional - backup path cost
                        y2( backup( m ) ) = y2( backup( m ) ) + d( i, 3 );
                        backup_paths_DP( i, m ) = backup( m );
                    end
                end
                
                if protection == 3
                    backup_paths_SP( i, : ) = zeros( 1, length( A ) );
                    for m = 1 : length( backup )
                        y3_SP{ backup( m ) } = [ y3_SP{ backup( m ) } d( i, 3 ) ];    % optional - backup path cost
                        backup_paths_SP( i, m ) = backup( m );
                    end
                end
                
            end
        end
        
        
        % dedicated local protection
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        y2_DL = zeros( length( A ), 1 );

        for i = 1 : length( A )
            A( i, 6 ) = 0;
            [ cost, backup_loc ] = dijkstra( A, links, y, A( i, 1 ), A( i, 2 ), y_DL( i ) );
            A( i, 6 ) = 1;

            backup_paths_DL( i, : ) = zeros( 1, length( A ) );
            for m = 1 : length( backup_loc )
                y2_DL( backup_loc( m ) ) = y2_DL( backup_loc( m ) ) + y_DL( i );    % optional - backup path cost
                backup_paths_DL( i, m ) = backup_loc( m );
            end
        end

        y = y + y2_DL;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        

        % shared global protection
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %y3_SP = cell( length( A ), 1 );

        for i = 1 : length( A )
            [ r, ~ ] = find( Pd == i );
            XXX = zeros( length( d ), length( A ) );
            XXX( r, : ) = backup_paths_SP( r, : );

            for j = 1 : length( A )
                [ r, ~ ] = find( XXX == j );

                y3_SP{ j } = [ y3_SP{ j } sum( d( r, 3 ) ) ];
            end
        end
        
        y2_SP = zeros( length( A ), 1 );

        for i = 1 : length( y2_SP )
            y2_SP( i ) = max( y3_SP{ i } );
        end

        y = y + y2_SP;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
        
        % shared local protection
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        y2_SL = zeros( length( A ), 1 );

        for i = 1 : length( A )
            A( i, 6 ) = 0;
            [ cost, backup_loc ] = dijkstra( A, links, y, A( i, 1 ), A( i, 2 ), y_SL( i ) );
            A( i, 6 ) = 1;

            backup_paths_SL( i, : ) = zeros( 1, length( A ) );
            for m = 1 : length( backup_loc )
                y3_SL{ backup_loc( m ) } = [ y3_SL{ backup_loc( m ) } y_SL( i ) ];    % optional - backup path cost
                backup_paths_SL( i, m ) = backup_loc( m );
            end
        end

        for i = 1 : length( y2_SL )
            y2_SL( i ) = max( y3_SL{ i } );
        end

        y = y + y2_SL;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		% Compute the global cost of the network
		
		F = F_new;
		F_new = global_cost( Cx, Cy, y );
        disp( F_new );
        y_sum = sum( y > 0 );
        disp( y_sum );
        
        % For each link, compute the derivatives (link unit costs)

		for i = 1 : length( A )
			A( i, 3 ) = get_avg_derivative_of_cost( Cx( i, : ), Cy( i, : ), y( i ), index, k );
		end
        
       % Determine whether another iteration is needed
		
        if F_new == F || k == k_max %( protection == 2 && k == 1 ) || ( protection == 3 && k == 2 ) || ( protection == 4 && k == 0 )
			
            if protection == 0
                uniq = unique( Pd );
                sleep = 1 : length( A );
                sleep = setdiff( sleep, uniq );
            end
            
            % STOP
			break;
        end

		k = k + 1;
    end
    
	fprintf( 'k = %d\n\n', k );

    
%% SAVE THE SELECTED VARIABLES INTO A .mat FILE

    save( 'output_NR', 'A', 'd', 'nodes', 'links', 'Cx', 'Cy', 'y', 'Pd' );
    Pd_DP = Pd;
    save( 'output_DP', 'A', 'd', 'nodes', 'links', 'Cx', 'Cy', 'y', 'Pd_DP', 'backup_paths_DP' );
    Pd_DL = Pd;
    save( 'output_DL', 'A', 'd', 'nodes', 'links', 'Cx', 'Cy', 'y', 'Pd_DL', 'backup_paths_DL' );
    Pd_SP = Pd;
    save( 'output_SP', 'A', 'd', 'nodes', 'links', 'Cx', 'Cy', 'y', 'y2_SP', 'Pd_SP', 'backup_paths_SP' );
    Pd_SL = Pd;
    save( 'output_SL', 'A', 'd', 'nodes', 'links', 'Cx', 'Cy', 'y', 'y2_SL', 'Pd_SL', 'backup_paths_SL' );
end    