/*********************************************
 * OPL 12.6.0.0 Model
 * Author: cholda
 * Creation Date: 21 May 2014 at 15:03:30
 *********************************************/
 
 float BigM = ...; // a very big number: big-M

 float restoration_coefficient = ...; // percentage of the working capacity put as backup

 float maxRisk = ...; // maximum acceptable risk level for the whole network
 
 float maxBudget = ...; // maximum acceptable budget level for the whole network

 float coefficient_energy = ...; // monetary cost of a unit of capacity
 
 float Energy_baseline = ...; // energy for provisioning non recovered connections
 
 float coefficient_risk = ...; // monetary cost of a unit of risk
 
 float Risk_baseline = ...; // the level of risk for non recovered connections

 {string} RecoveryMethod = ...; // $t = 1,2,\dots,T$ used recovery method (in fact, t = {NR; DPP; DLP; SBPP; Re-routing50%})

 {string} CompensationPolicy = ...; // the used compensation policy: continuity or availability
 
 {string} RiskMeasure = ...; // the used risk measure: RE, VaR or CVaR

 {string} Nodes = ...; // network nodes
 
 tuple arc // $e = 1,2,\dots,E$ network links (edges)
 {
   string source;
   string destination;
 }   
 
 {arc} Arcs with source in Nodes, destination in Nodes = ...;

 //float Capacity[Arcs] = ...;
 
 //float Cost[Arcs] = ...;
 
 tuple demand // $d = 1,2,\dots,D$ demands (connections)
 {
   string source;
   string destination;
 }   
 
 {demand} Demands with source in Nodes, destination in Nodes = ...;
 
 //float risk[RecoveryMethod][Demands][CompensationPolicy][RiskMeasure] = ...; // predicted value of risk incurred for demand d when it uses recovery method t with a given risk measure and compensation policy
 
 float risk[RecoveryMethod][Demands] = ...; // predicted value of risk incurred for demand d when it uses recovery method t with a given risk measure and compensation policy r_dt
  
 float energy_usage[RecoveryMethod][Demands] = ...; // share in the energy usage for demand d when it uses recovery method t e_dt
 
 dvar float+ Total_energy; // total energy used in the network
 
 dvar float Involved_budget; // the monetary cost of providing additional energy for risk mitigation [B]
 
 dvar float+ Total_risk; // total risk in the network (expressed in monetary units) [R]
 
 dvar float+ Risk_decrease; // decrease of risk in comparison to the baseline risk [D]
 
 dvar boolean recovery_method[RecoveryMethod][Demands]; // = 1 if demand d uses recovery method t, = 0, otherwise [x_dt]
 
 dvar int number_recovery[RecoveryMethod]; // informs how many times a recovery method is assigned to connections
 
 //float Volume[Demands] = ...; // volume of demand d to be realized
 
 //int Path = ...;
 
 //range Paths = 1..Path; //$p = 1,2,dots,P_d$ candidate paths for flows realizing demand d
 
 //int Segment = ...;
 
 //range Segments = 1..Segment; //$q = 1,2,dots,Q_e$ candidate segments for backups of link e
 
// int delta[Arcs][Demands][Paths] = ...; // = 1 if e belongs to candidate working path p realizing demand d, = 0, otherwise
 
 //int beta[Arcs][Demands][Paths] = ...; // = 1 if e belongs to candidate backup path p realizing demand d, = 0, otherwise (for a given demand d, the pair of the working and demand paths have disjoint set of links)
  
 //int gamma[Arcs][Arcs][Paths] = ...; // = 1 if e belongs to candaidate segment p realizing backup for are a, = 0, otherwise
  
 //dvar boolean flow[RecoveryMethod][Demands][Paths]; // = 1 if demand d uses path p with recovery method t, = 0, otherwise
 
 //dvar float+ flows_working[RecoveryMethod][Arcs]; // amount of capacity allocated in link e for working resources using recovery method t

 //dvar float+ flows_backup[RecoveryMethod][Arcs]; // amount of capacity allocated in link e for backup resources using recovery method t

// int flow_dlp_which[Arcs][Segments]; // = 1 if segment q is used to recover with DLP capacity of arc e
 
 //int flow_dlp[Arcs][Segments]; // capacity protecting arc e at segment q for DLP
 
 //int flow_slp_which[Arcs][Segments]; // = 1 if segment q is used to recover with SLP capacity of arc e
 
 //int flow_slp[Arcs][Segments]; // capacity protecting arc e at segment q for SLP
 
 //dvar float+ flow_summarized[RecoveryMethod][Arcs]; // amount of capacity allocated in link e for working and backup resources using recovery method t;

 // Profit maximization:
 //minimize Involved_budget + Total_risk;
 
 // Risk minimization and cost balance:
 minimize Total_risk;

 // Total benefit coverage:
 //maximize Risk_decrease;

 // Risk acceptance:
 //minimize Involved_budget;
 
 subject to{

  // Risk acceptance
  //forall(d in Demands)
    //recovery_method["NoRecovery"][d] == 1;

  forall(d in Demands)
     sum(rm in RecoveryMethod) recovery_method[rm][d] == 1; // a demand uses only one recovery method
     
  //forall(rm in RecoveryMethod)
     //number_recovery[rm] == sum(d in Demands) recovery_method[rm][d];

  Total_energy == sum(rm in RecoveryMethod,d in Demands) coefficient_energy*energy_usage[rm][d]*recovery_method[rm][d];
  
  Involved_budget == Total_energy - coefficient_energy*Energy_baseline;
  
  Total_risk == sum(rm in RecoveryMethod, d in Demands) coefficient_risk*risk[rm][d]*recovery_method[rm][d];
    
  Risk_decrease == coefficient_risk*Risk_baseline - Total_risk;

  // Total benefit coverage:
  //Involved_budget <= Risk_decrease;
   
  // Cost balance:
  //Involved_budget <= Total_risk;
 
 } 
 
 execute{
   var ofile = new IloOplOutputFile("results_summarized.txt",true);
   
   ofile.writeln("/////////////////////////////////////////////////////////////////////////////////////////////");
   
   ofile.writeln("TotalBenefitCoverage-",coefficient_energy,"-",coefficient_risk);
   
   /*
   ofile.writeln("Nodes = ",Nodes);
   ofile.writeln("Arcs = ",Arcs);
   ofile.writeln("Demands = ",Demands);
   */
    
   //writeln(basel); 
    	 
   ofile.writeln("Goal function = ",cplex.getObjValue()," mln USD.");
   ofile.writeln("Total energy = ",Total_energy," mln USD.");
   // writeln("Baseline energy = ",Energy_baseline*coefficient_energy," mln USD.");
   ofile.writeln("Involved budget = ",Involved_budget," mln USD.");
   ofile.writeln("Total risk = ",Total_risk," mln USD.");
   // writeln("Risk baseline = ",Risk_baseline*coefficient_risk," mln USD.");
   ofile.writeln("Risk decrease = ",Risk_decrease," mln USD.");
      
   ofile.writeln("");

   for(var rm in RecoveryMethod)
   	ofile.writeln(rm," assigned ",number_recovery[rm]," times.");
   	
   ofile.writeln("");
   
   for(var d in Demands)
   	for(var rm in RecoveryMethod)
   		if(recovery_method[rm][d] > 0)
   		{
   			ofile.writeln("Demand ",d," uses ",rm,".");
   			ofile.write("Risk: ",risk[rm][d],": ");
   			for(var rec in RecoveryMethod)
   				ofile.write(risk[rec][d]," ");
   			ofile.writeln();
   			ofile.write("Energy: ",energy_usage[rm][d],": ");
   			for(var rec in RecoveryMethod)
   				ofile.write(energy_usage[rec][d]," ");
   			ofile.writeln();
   			ofile.write("Risk/energy: ",coefficient_risk*risk[rm][d]/(coefficient_energy*energy_usage[rm][d]),": ")
   			for(var rec in RecoveryMethod)
   				ofile.write(coefficient_risk*risk[rec][d]/(coefficient_energy*energy_usage[rec][d])," ")
   			ofile.writeln();
    	}   			   			

   ofile.writeln();

   ofile.close();
   
   var secondfile = new IloOplOutputFile("result_table.txt",true);
   
   secondfile.writeln("TotalBenefitCoverage:");
   
   secondfile.writeln(coefficient_energy,"-",coefficient_risk,",",number_recovery["NoRecovery"],"-",number_recovery["DedicatedPathProtection"],"-",number_recovery["DedicatedLinkProtection"],"-",number_recovery["SharedBackupPathProtection"],"-",number_recovery["SharedBackupLinkProtection"],",",Total_risk,",",Involved_budget,",",Total_risk+Involved_budget,",",Risk_decrease,",",Total_energy,",");
   
   secondfile.close();
   
   var thirdfile = new IloOplOutputFile("../modul_symulacyjny/start.m");
   
   
   thirdfile.write("demands = [");
   
   for(var d in Demands)
   	for(var rm in RecoveryMethod)
   		if(recovery_method[rm][d] > 0)
   		{
   			//thirdfile.write(" ",d,rm,": ");
   			if (rm == "NoRecovery")
   				thirdfile.write(" 0 ");
   			else if (rm == "DedicatedPathProtection")
   				thirdfile.write(" 1 ");
   			else if (rm == "DedicatedLinkProtection")
   				thirdfile.write(" 2 ");
   			else if (rm == "SharedBackupPathProtection")
   				thirdfile.write(" 3 ");
   			else if (rm == "SharedBackupLinkProtection")
   				thirdfile.write(" 4 ");
    	}   
    	
   thirdfile.writeln("];");
   thirdfile.writeln("save( 'demands', 'demands' )");
   thirdfile.writeln("cd ../modul_optymalizacyjny/");
   thirdfile.writeln("main( demands );");
   thirdfile.writeln("cd ../modul_symulacyjny/");
   thirdfile.writeln("%main( 8, 10000 );");
   
   /*thirdfile.write("risk = [")
   for(var rm in RecoveryMethod)
   {
   	 thirdfile.write("[");
   	 for(var d in Demands)
   	    thirdfile.write(" ",risk[rm][d]," ");	 
   	 thirdfile.writeln("]");
   }
   thirdfile.writeln("];");
   thirdfile.writeln();

   thirdfile.write("energy_usage = [")
   for(var rm in RecoveryMethod)
   {
   	 thirdfile.write("[");
   	 for(var d in Demands)
   	    thirdfile.write(" ",energy_usage[rm][d]," ");	 
   	 thirdfile.writeln("]");
   }
   thirdfile.writeln("];");*/
   			   			
   thirdfile.close();

 }