%% GENERAL INFORMATION
% Discrete-time simulator module.
%
% Author: Piotr Jaglarz (pjaglarz@student.agh.edu.pl)
% Date: 04.2014

%% INPUT

% Clear the environment

% 	close all;
% 	clear all;
% 	clc;

function [ total_d, number_d, wake_time, wake_number, wake_fail, sim_time, m_link_load, m_link_load_backup, m_Global_cost, m_Backup_cost, m_Path_cost ] = main_simulation( N, distribution, demands, i_los )

% Generate a data set for the discrete-time simulator module.

    % [ id, timeline, timeline_add, filename ] = gen_input_data( N );
    [ ~, timeline, timeline_add, ~ ] = gen_input_data( N, distribution, i_los );

    
% Recovery meteod (0-off, 1-prot_ded_glob, 2-prot_ded_loc, 3-prot_sh_glob, 4-prot_sh_loc, 5-global_restoration, 6-local_restoration)
    

%% SIMULATION

    % Read the complete environment snapshot from a .mat file
    % time = clock;
    sleep = 0;
    %switch recovery_metod
    %    case 0
            load( '../modul_optymalizacyjny/output_NR' );
    %    case 1
            load( '../modul_optymalizacyjny/output_DP' );
    %    case 2
            load( '../modul_optymalizacyjny/output_DL' );
    %    case 3
            load( '../modul_optymalizacyjny/output_SP' );
    %    case 4
            load( '../modul_optymalizacyjny/output_SL' );

    
	% Downtime array for connections (traffic demands) & connections availability matrix
	
	downtime = zeros( 2, length( d ) );
    wakeup = zeros( 2, length( A ) );
    wake_failure = zeros( length( d ), 1 );
	%connections_availability =  true( length( d ), length( timeline ) ) ;
    link_load = zeros( length( A ), 1 );
    % link_load_pr = zeros( length( A ), 1 );
    link_load_ba = zeros( length( A ), 1 );
    Global_cost = 0;
    Backup_cost = 0;
	Path_cost = zeros( length( d ), 1 );
    

    % Main loop
	
	for i = 1 : length( timeline ) - 1

        % Matrix declaration
        Pd_now = zeros( length( d ), length( A ) );
        link_load_primary = zeros( length( A ), 1 );
        link_load_backup = zeros( length( d ), length( A ) );
        backup_link_load = zeros( length( A ), 1 );
        backup_link_load1 = zeros( length( A ), 1 );
        
        
		% Iterate through the 'events' array to perform required actions
		% for links that either went up or down (or any other valid status) at timeline( i )

        for k = 1 : size( timeline_add, 2 )
            if timeline_add( i, k ) == 0
                break;
            end

            j = timeline_add( i, k );

            if A( j, 6 ) == 0
                A( j, 6 ) = 1;
            elseif A( j, 6 ) == 1
                A( j, 6 ) = 0;
            elseif A( j, 6 ) == 2
                A( j, 6 ) = 20;
            elseif A( j, 6 ) == 20
                A( j, 6 ) = 2;
            end

        end

		       
        vectord = randperm( length( d ) );
        for j = vectord
			path_ok = true;
            
            recovery_metod = demands( j );
            
			% Check status of the working path
            switch recovery_metod
                case 0
                    for k = 1 : length( A )
                        if Pd( j, k ) == 0
                            % Padding detected - interrupt processing

                            break;
                        end

                        if A( Pd( j, k ), 6 ) == 0
                            % Link failure detected, try to find a suitable backup path..
                            path_ok = false;

                            break;
                        end
                    end
                case 1
                    for k = 1 : length( A )
                        if Pd_DP( j, k ) == 0
                            % Padding detected - interrupt processing

                            break;
                        end

                        if A( Pd_DP( j, k ), 6 ) == 0
                            % Link failure detected, try to find a suitable backup path..
                            path_ok = false;

                            break;
                        end
                    end
                case 2
                    for k = 1 : length( A )
                        if Pd_DL( j, k ) == 0
                            % Padding detected - interrupt processing

                            break;
                        end

                        if A( Pd_DL( j, k ), 6 ) == 0
                            % Link failure detected, try to find a suitable backup path..
                            path_ok = false;

                            break;
                        end
                    end
                case 3
                    for k = 1 : length( A )
                        if Pd_SP( j, k ) == 0
                            % Padding detected - interrupt processing

                            break;
                        end

                        if A( Pd_SP( j, k ), 6 ) == 0
                            % Link failure detected, try to find a suitable backup path..
                            path_ok = false;

                            break;
                        end
                    end
                case 4
                    for k = 1 : length( A )
                        if Pd_SL( j, k ) == 0
                            % Padding detected - interrupt processing

                            break;
                        end

                        if A( Pd_SL( j, k ), 6 ) == 0
                            % Link failure detected, try to find a suitable backup path..
                            path_ok = false;

                            break;
                        end
                    end
            end
			
			% Backup paths - check only if needed
			
            if path_ok == false
                
                switch recovery_metod
                    case 1
                        path = [];
                        for z = 1 : length( A )
                            if backup_paths_DP( j, z ) == 0
                                % Padding detected - interrupt processing (we have found a backup path)
                                path_ok = true;

                                % Increase the link load
                                for s = 1 : length( path )
                                    link_load_backup( j, path( s ) ) = link_load_backup( j, path( s ) ) + d( j, 3 );
                                    Pd_now( j, s ) = path( s );
                                end
                                
                                break;
                            end

                            if A( backup_paths_DP( j, z ), 6 ) == 0
                                % Link failure detected, there is no working backup path..

                                break;
                            end
                            path = [ path backup_paths_DP( j, z ) ];
                        end
                        
                    case 2
                        paths = [];
                        for k = 1 : length( A )
                            if Pd_DL( j, k ) == 0
                                path_ok = true;

                                % Increase the link load
                                for s = 1 : length( paths )
                                    link_load_backup( j, paths( s ) ) = link_load_backup( j, paths( s ) ) + d( j, 3 );
                                    Pd_now( j, s ) = paths( s );
                                end

                                break;
                            end

                            if A( Pd_DL( j, k ), 6 ) == 0
                                path_fail = false;
                                path = [];
                                for z = 1 : length( A )
                                    if backup_paths_DL( Pd_DL( j, k ), z ) == 0
                                        % Padding detected - interrupt processing

                                        break;
                                    end

                                    if A( backup_paths_DL( Pd_DL( j, k ), z ), 6 ) == 0
                                        % Link failure detected, there is no working backup path..
                                        path_fail = true;
                                        break;
                                    end
                                    path = [ path backup_paths_DL( Pd_DL( j, k ), z ) ];
                                end

                                if path_fail
                                    break;
                                else
                                    paths = [ paths path ];
                                end
                                
                            else
                                paths = [ paths Pd_DL( j, k ) ];
                            end
                        end
                        
                    case 3
                        path = [];
                        for z = 1 : length( A )
                            if backup_paths_SP( j, z ) == 0
                                % Padding detected - interrupt processing (we have found a backup path)
                                path_ok = true;

                                % Increase the link load
                                for s = 1 : length( path )
                                    link_load_backup( j, path( s ) ) = link_load_backup( j, path( s ) ) + d( j, 3 );
                                    Pd_now( j, s ) = path( s );
                                end
                                
                                break;
                            end

                            if A( backup_paths_SP( j, z ), 6 ) == 0 || backup_link_load( backup_paths_SP( j, z ) ) + d( j, 3 ) > y2_SP( backup_paths_SP( j, z ) )
                                % Link failure detected, there is no working backup path..

                                break;
                            end
                            path = [ path backup_paths_SP( j, z ) ];
                            backup_link_load( backup_paths_SP( j, z ) ) = backup_link_load( backup_paths_SP( j, z ) ) + d( j, 3 );
                        end
                        
                    case 4
                        paths = [];
                        for k = 1 : length( A )
                            if Pd_SL( j, k ) == 0
                                path_ok = true;

                                % Increase the link load
                                for s = 1 : length( paths )
                                    link_load_backup( j, paths( s ) ) = link_load_backup( j, paths( s ) ) + d( j, 3 );
                                    Pd_now( j, s ) = paths( s );
                                end

                                break;
                            end

                            if A( Pd_SL( j, k ), 6 ) == 0
                                path_fail = false;
                                path = [];
                                for z = 1 : length( A )
                                    if backup_paths_SL( Pd_SL( j, k ), z ) == 0
                                        % Padding detected - interrupt processing

                                        break;
                                    end

                                    if A( backup_paths_SL( Pd_SL( j, k ), z ), 6 ) == 0 || backup_link_load1( backup_paths_SL( Pd_SL( j, k ), z ) ) + d( j, 3 ) > y2_SL( backup_paths_SL( Pd_SL( j, k ), z ) )
                                        % Link failure detected, there is no working backup path..
                                        path_fail = true;
                                        break;
                                    end
                                    path = [ path backup_paths_SL( Pd_SL( j, k ), z ) ];
                                    backup_link_load1( backup_paths_SL( Pd_SL( j, k ), z ) ) = backup_link_load1( backup_paths_SL( Pd_SL( j, k ), z ) ) + d( j, 3 );
                                end

                                if path_fail
                                    break;
                                else
                                    paths = [ paths path ];
                                end
                                
                            else
                                paths = [ paths Pd_SL( j, k ) ];
                            end
                        end
                        
                end
                
                % We have not found any suitable backup path --> increase downtime
                if path_ok == false
										
					%connections_availability( j, i ) = 0;

                    downtime( 1, j ) = downtime( 1, j ) + ( timeline( i + 1 ) - timeline( i ) );
                    downtime( 2, j ) = downtime( 2, j ) + 1;

                end               
            
            else
                % Primary path is OK
                switch recovery_metod
                    case 0
                        for k = 1 : length( A )
                            if Pd( j, k ) ~= 0
                                link_load_primary( Pd( j, k ) ) = link_load_primary( Pd( j, k ) ) + d( j, 3 );
                                Pd_now( j, k ) = Pd( j, k );
                            else
                                break;
                            end
                        end
                    case 1
                        for k = 1 : length( A )
                            if Pd_DP( j, k ) ~= 0
                                link_load_primary( Pd_DP( j, k ) ) = link_load_primary( Pd_DP( j, k ) ) + d( j, 3 );
                                Pd_now( j, k ) = Pd_DP( j, k );
                            else
                                break;
                            end
                        end
                    case 2
                        for k = 1 : length( A )
                            if Pd_DL( j, k ) ~= 0
                                link_load_primary( Pd_DL( j, k ) ) = link_load_primary( Pd_DL( j, k ) ) + d( j, 3 );
                                Pd_now( j, k ) = Pd_DL( j, k );
                            else
                                break;
                            end
                        end
                    case 3
                        for k = 1 : length( A )
                            if Pd_SP( j, k ) ~= 0
                                link_load_primary( Pd_SP( j, k ) ) = link_load_primary( Pd_SP( j, k ) ) + d( j, 3 );
                                Pd_now( j, k ) = Pd_SP( j, k );
                            else
                                break;
                            end
                        end
                    case 4
                        for k = 1 : length( A )
                            if Pd_SL( j, k ) ~= 0
                                link_load_primary( Pd_SL( j, k ) ) = link_load_primary( Pd_SL( j, k ) ) + d( j, 3 );
                                Pd_now( j, k ) = Pd_SL( j, k );
                            else
                                break;
                            end
                        end
                end
                
            end
            link_load_now = link_load_primary + sum( link_load_backup )';
        end
        
        
        delta = timeline( i + 1 ) - timeline( i );

        Global_cost = Global_cost + delta * global_cost( Cx, Cy, link_load_now );
        Path_cost = Path_cost + delta * path_cost( Cx, Cy, Pd_now, link_load_now, d( :, 3 ) );
        for j = 1 : length( d )
            backup_cost_now( j, : ) = sqrt( link_load_now ) .* link_load_backup( j, : )' ./ link_load_now;
        end
        backup_cost_now( isnan( backup_cost_now ) ) = 0;
        Backup_cost = Backup_cost + delta * sum( backup_cost_now, 2 );
        link_load = link_load + delta * link_load_now;
        % link_load_pr = link_load_pr + link_load_primary;
        link_load_ba = link_load_ba + delta * sum( link_load_backup )';
	end
	
	
    sim_time = timeline( end );
    total_d = 10 * 365 * 24 * downtime( 1, : )' ./ sim_time;
    number_d = 10 * 365 * 24 * downtime( 2, : )' ./ sim_time;
    wake_time = 10 * 365 * 24 * wakeup( 1, : )' ./ sim_time;
    wake_number = 10 * 365 * 24 * wakeup( 2, : )' ./ sim_time;
    wake_fail = 10 * 365 * 24 * wake_failure ./ sim_time;
    m_link_load = link_load ./ sim_time;
    m_link_load_backup = link_load_ba ./ sim_time;
    m_Global_cost = Global_cost / sim_time;
    m_Backup_cost = Backup_cost / sim_time;
    m_Path_cost = Path_cost ./ sim_time;
    

%% SAVE VARIABLES INTO A .mat FILE

    % save( test.m, 'downtime', 'd', 'Pd', 'sleep', 'connections_availability', 'link_load', 'timeline', 'Global_cost', 'Path_cost' );
    
