clear all

method = 2;
%opt_method = 'risk minimization';
%opt_method = 'total benefit coverage';
%opt_method = 'cost balance';
opt_method = 'profit maximization';
risk_coef = 2;
network = 'nobel-us';

if method == 2
    method_str = 'In each simulation iteration, the {\bffull optimization} with the Yageds heuristic is done.';
    method_str2 = 'full optimization';
elseif method == 4
    method_str = 'In each simulation iteration (except for the first iteration), the initial Yageds heuristic sets only a {\bflimit for the shared backup} capacity.';
    method_str2 = 'limit for the shared backup';
end

caption2 = sprintf('%s  %s  coefficient_%d', opt_method, method_str2, risk_coef);
caption = strrep( caption2, ' ', '_' );

list = dir;

k = 1;

for i = 1 : length( list )
    if strcmp( list( i ).name, 'SUM.mat' )
        continue;
    end
    if strfind( list( i ).name, 'mat' )
        load( list( i ).name );
        total_risk( k ) = mean( d( :, 3 )' * total_down );
        err( k ) = 1.96 * std( d( :, 3 )' * total_down ) / sqrt( length( simulation_time ) );
        total_energy( k ) = mean( Global_cost );
        err_en( k ) = 1.96 * std( Global_cost ) / sqrt( length( simulation_time ) );
        
        for j = 0 : 4
            total_demands( k, j+1 ) = sum( demands == j );
        end
        
        k = k + 1;
    end
end

summ = [ total_energy; total_risk; err ];

save( 'SUM', 'summ', 'total_risk', 'total_energy', 'err', 'err_en' );

hFig = figure;
%set(gcf,'PaperPositionMode','auto')
set(hFig, 'Position', [400 200 1000 700])
subplot( 3, 1, 1 )
hold on
plot( summ( 1, : ), 'k.-', 'LineWidth', 1.3 )
%errorbar( summ( 1, : ), err_en, 'k' )
box on
grid on
set(gca, 'GridLineStyle', '--')
xlim([0 length(summ)+1])
xlabel('\bfIteration')
ylabel('\bfEnergy usage')

subplot( 3, 1, 2 )
hold on
plot( summ( 2, : ), 'k.-', 'LineWidth', 1.3 )
errorbar( summ( 2, : ), err, 'k' )
box on
grid on
set(gca, 'GridLineStyle', '--')
xlim([0 length(summ)+1])
xlabel('\bfIteration')
ylabel('\bfRisk')

subplot( 3, 1, 3 )
plot( total_demands, '.-', 'LineWidth', 1 )
grid on
set(gca, 'GridLineStyle', '--')
xlim([0 length(summ)+1])
legend( 'NR', 'DP', 'DL', 'SP', 'SL' )
xlabel('\bfIteration')
ylabel({'\bfNumber of assigned';'recovery options'})

suptitle({sprintf('\\fontsize{10}{\\bfnetwork: %s}', network);...
    sprintf('\\fontsize{8}Results of the {\\bf%s} optimization obtained with the iterated algorithm.', opt_method);...
    method_str;...
    sprintf('Monetary coefcient of penalty (risk) to energy usage is {\\bf%d}. The 95%% confidence intervals are shown for risk results.', risk_coef)})

saveas(hFig, caption, 'png');


%% Zapis pliku
fid = fopen( sprintf( '%s.tex', caption ), 'wt' );

fprintf( fid, '\\documentclass{article}\n\\usepackage{pgfplots}\n\n\n\\begin{document}\n\n' );

% Energy
fprintf( fid, '\\begin{figure}\n' );
    fprintf( fid, '\t\\begin{tikzpicture}\n' );
        fprintf( fid, '\t\t\\begin{axis}[xlabel={Number of iterations}, ylabel={Energy usage}]\n' );%[ybar, bar width=5pt, ymin=0, xmin=0.5,xmax=%d.5, xlabel={OPIS_OSI_X}, ylabel={OPIS_OSI_Y},ylabel style={yshift=-0.5cm},xtick={1,2,3,4} , xticklabels={100,200,350,500},xtick style={color=white, align=inside}, height=4cm,width=\\columnwidth]\n\n', length( summ ) );

            %fprintf( fid, '\t\t\t\\addplot+[black,pattern=horizontal lines, pattern color=gray, error bars/.cd,y dir=both,y explicit]  coordinates {' );
            fprintf( fid, '\n\t\t\t\\addplot coordinates {' );
            for i = 1 : length( summ )
                fprintf( fid, '(%d,%.2f) +- (%.2f,%.2f) ', i, total_energy( i ), err_en( i ), err_en( i ) );
            end
            fprintf( fid, '};\n\n' );
        
        fprintf( fid, '\t\t\\end{axis}\n' );
    fprintf( fid, '\t\\end{tikzpicture}\n' );
    fprintf( fid, '\t\\caption{%s} \\label{fig:ISP-break}\n', caption2 );
fprintf( fid, '\\end{figure}\n' );

% Risk
fprintf( fid, '\n\\begin{figure}\n' );
    fprintf( fid, '\t\\begin{tikzpicture}\n' );
        fprintf( fid, '\t\t\\begin{axis}[xlabel={Number of iterations}, ylabel={Risk}]\n' );%[ybar, bar width=5pt, ymin=0, xmin=0.5,xmax=%d.5, xlabel={OPIS_OSI_X}, ylabel={OPIS_OSI_Y},ylabel style={yshift=-0.5cm},xtick={1,2,3,4} , xticklabels={100,200,350,500},xtick style={color=white, align=inside}, height=4cm,width=\\columnwidth]\n\n', length( summ ) );

            %fprintf( fid, '\t\t\t\\addplot+[black,pattern=horizontal lines, pattern color=gray, error bars/.cd,y dir=both,y explicit]  coordinates {' );
            fprintf( fid, '\n\t\t\t\\addplot coordinates {' );
            for i = 1 : length( summ )
                fprintf( fid, '(%d,%.2f) +- (%.2f,%.2f) ', i, total_risk( i ), err( i ), err( i ) );
            end
            fprintf( fid, '};\n\n' );
        
        fprintf( fid, '\t\t\\end{axis}\n' );
    fprintf( fid, '\t\\end{tikzpicture}\n' );
    fprintf( fid, '\t\\caption{%s} \\label{fig:ISP-break}\n', caption2 );
fprintf( fid, '\\end{figure}\n' );

% Demands
fprintf( fid, '\n\\begin{figure}\n' );
    fprintf( fid, '\t\\begin{tikzpicture}\n' );
        fprintf( fid, '\t\t\\begin{axis}[xlabel={Number of iterations}, ylabel={Number of demands}]\n' );%[ybar, bar width=5pt, ymin=0, xmin=0.5,xmax=%d.5, xlabel={OPIS_OSI_X}, ylabel={OPIS_OSI_Y},ylabel style={yshift=-0.5cm},xtick={1,2,3,4} , xticklabels={100,200,350,500},xtick style={color=white, align=inside}, height=4cm,width=\\columnwidth]\n\n', length( summ ) );

            for j = 1 : 5
                fprintf( fid, '\n\t\t\t\\addplot coordinates {' );
                for i = 1 : length( summ )
                    fprintf( fid, '(%d,%.2f) +- (%.2f,%.2f) ', i, total_demands( i, j ), 0,0 );
                end
                fprintf( fid, '};\n\n' );
            end
            fprintf( fid, '\t\t\t\\legend{NR,DP,DL,SP,SL}\n\n' );
        
        fprintf( fid, '\t\t\\end{axis}\n' );
    fprintf( fid, '\t\\end{tikzpicture}\n' );
    fprintf( fid, '\t\\caption{%s} \\label{fig:ISP-break}\n', caption2 );
fprintf( fid, '\\end{figure}\n' );

fprintf( fid, '\n\\end{document}\n' );

fclose( fid );